<?php
/*#################################################################
#
# Name:	common.php
#
# Description:
#   Routines common to generate headers etc
#
# Copyright (C) 2004-2005 Terry Gliedt, University of Michigan
# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; See http://www.gnu.org/copyleft/gpl.html
#################################################################*/
//  Be sure I can see syntax errors
//ini_set('error_reporting', E_ALL ^ E_NOTICE);
$VERSION = '0.99';              // Version of this release of SWS

//  Try to deal with ancient forms of PHP
$v = phpversion();
if ($v < 4.1) {
    $_SERVER = $HTTP_SERVER_VARS;
    $_SESSION = $HTTP_SESSION_VARS;
    $_POST = $HTTP_POST_VARS;
    $_GET = $HTTP_GET_VARS;
}
$REVISIONDATE = '';                 // Date of PHP or HTML file

global $SWSDIR;
include_once "$SWSDIR/local_config.php";    // Define site, same dir as common.php

/*---------------------------------------------------------------
# do_page - Generate an HTML page using all the default layout
#
# Parameters:
#   title - optional title
#   centerhtml - optional HTML to show in center panel
#
# Returns:
#   nothing
---------------------------------------------------------------*/
function do_page($title='No title provided', $centerhtml = '') {
    global $LHMENUS, $RHMENUS, $RHMENUS_IF_AUTH, $HEADERLINES,
         $KEYWORDS, $JSCRIPT;

    //  Get things from any session that was set up
    session();

    //  If no HTML was provided, get HTML from an inferred file
    //  and get possible meta keywords and jscript/style lines
    $f = '';
    if (! $centerhtml) {
        list($centerhtml, $keywords) = do_center();
        $KEYWORDS .= ' ' . $keywords;
    }

   //  Generate a page, using our default page construction
    $hlines = '';
    if ($KEYWORDS) { $hlines .= "<meta name=\"keywords\" content=\"$KEYWORDS\"/>\n"; }
    do_header($title, $HEADERLINES . $hlines . $JSCRIPT);
    do_banner();

    //  We always have LH menus
    print "<!-- Generate page. Menus on left. Page in middle. Possibly menus on right -->\n" .
        "<table summary=\"2,3 columns\">\n<tr>  <td valign=\"top\" class=\"leftcol\">\n";
    do_staticnav($LHMENUS);
    print "  </td>\n";

    //  Show principal page here. HTML will either be inferred, based
    //  on the name of this PHP file, or we'll use HTML provided by caller
    print "  <td valign=\"top\" class=\"middlecol\">\n";
    print $centerhtml;
    print "</td>\n";

    //  Show RH menus if a file is provided.
    //  Maybe only show them if we are authenticated
    if ($RHMENUS) {
        if ((! $RHMENUS_IF_AUTH) ||
            ($RHMENUS_IF_AUTH && $_SESSION['SESSTIME'])) {
            print "<td valign=\"top\" class=\"rightcol\">\n";
            do_staticnav($RHMENUS);
            print "</td>\n";
        }
    }
    print "</tr></table>\n";

    do_footer($f);
    flush();                // Sourceforge did not flush!!
}

/*---------------------------------------------------------------
# do_header - Generate an HTML header
#
# Parameters:
#   title - optional title or use global variable $TITLE
#   more  - optional additional header lines (like javascript etc)
#           If this ends in '.inc', it is a file to be read
#           and put to STDOUT
#
# Returns:
#   nothing
---------------------------------------------------------------*/
function do_header($title='No title provided', $more='') {
    global $SITEJS, $TITLE, $VERSION, $STYLE;

    //  If $TITLE is provided, use that
    if ($TITLE) { $title = $TITLE; }

    //  Generate the header
    print <<<END
<?xml version="1.0" encoding="iso-8859-1"?>
<!DOCTYPE html 
     PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
     "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en">
<head>
<title>$title</title>
<meta http-equiv="Content-Type" content="text/html;charset=iso-8859-1"/>
<meta name="keywords" content="This uses SWS version $VERSION"/>
<!-- Style controls for look and feel of page -->

END;

    //  Normal style sheet. Reasonable sites only have one of these.
    print "<style type=\"text/css\">@import url(\"$SITEJS\");</style>\n";
    if ($STYLE) { print $STYLE; }
    //  Add user lines
    print "$more\n</head><body>\n";
}

/*---------------------------------------------------------------
# do_banner - Generate default banner for site
#
# Parameters:
#   none
#
# Returns:
#   none
---------------------------------------------------------------*/
function do_banner() {
    global $BANNERFILE;
    
    if (! $BANNERFILE) { return; }
    if (! file_exists($BANNERFILE)) { return; }
    
    if (! readfile($BANNERFILE)) {
        redmsg("do_banner: Unable to read file '$BANNERFILE'");
    }
}

/*---------------------------------------------------------------
# do_staticnav - Generate static menus
#
# Parameters:
#   file - file of menus to read
#       File has the following format:
#           END in col 1, pseudo EOF (useful for debugging)
#           # in col 1, comment
#           Blank lines are ignored
#           BLANKLINE in col 1, insert spacer in menu
#           BLOCK= in col 1 defines title of a menu block
#           Users authorized to see this menu are in [] after block title
#           Authorized user [*] means 'all authenticated users'
#           Menu link starts in col 1 with <a and is on one line
#           Indented menu link starts in col 1 with INDENT <a and is on one line
#           Anything else in col 1 is HTML which is inserted unchanged
#
# Examples:
#
#   BLOCK=See Me
#   <a class="menu-item" href="/Repository" target="_blank">My Data</a>
#   BLANKLINE
#   BLOCK=See Some of Me [fred deb]
#   <a class="menu-item" href="/Yours" target="_blank">Your Data</a>
#   BLOCK=See All of Me [*]
#   <a class="menu-item" href="/Ours" target="_blank">Our Data</a>
#   #   This block is everything from <form to </form
#   INDENT<a class="menus" href="/Ours" target=_blank>Our Data</a>
#   #   This block is everything from <form to </form
#   BLOCK=Search Here
#   <form method="get" action="/pubcgi/swish.cgi">
#     <input maxlength="200" size="18" type="text" name="query">
#     <input value="Search!" type="submit" name="submit">
#     <input value="fusion" type="hidden" name="use">
#     <input value="selected" type="hidden" name="swishrank'>
#   </form>
#
# Returns:
#   none
---------------------------------------------------------------*/
function do_staticnav($file) {
    global $BLOCKNAMEHTML, $MENUITEMHTML, $MENUINDENTHTML, $SHOWBLOCK, $_SESSION;

    $blocktop = "<table class=\"menus\">\n";
    $blockbottom = "<tr><td class=\"menus-spacer\"></td></tr>\n</table>\n<br/>\n";

    print "<!-- Static menus $file -->\n";
    $lines = file($file);
    $html = '';
    $blockname = '';
    for ($n=0; $n<count($lines); $n++) {
        $line = $lines[$n];
        if (preg_match('/^END/', $line)) { break; } // Pseudo EOF
        if (preg_match('/^#/', $line)) { continue; }
        if (preg_match('/^\s*$/', $line)) { continue; }

        if (preg_match('/^BLOCK=(.+)\s*$/', $line, $m)) {
            $noshow = 0;
            $blockname = $m[1];
            $authorizedids = array();
            //  Watch if special authentication is needed
            if (preg_match('/^(.+)\s*\[(.+)\]/', $blockname, $m)) {
                if (! $_SESSION['REMOTE_USER']) {     // Not authenticated
                    $blockname = '';
                    continue;
                }
                $blockname = $m[1];
                if ($m[2] != '*') {     // Not 'ALL USERS', check it out
                    $authorizedids = explode(' ',$m[2]);
                    //  If this menu is only for SOME users, see if this
                    //  person is allowed to see this
                    if (! in_array($_SESSION['REMOTE_USER'],$authorizedids)) {
                        $blockname = '';
                        continue;
                    }
                }
            }

            //  Watch if this BLOCK applies to only certain pages
            if (preg_match('/^(.+)\s+SHOWONLY=(\w+)\s*/', $blockname, $m)) {
                if ($SHOWBLOCK != $m[2]) {  // Skip this block
                    $blockname = '';
                    continue;
                }
                $blockname = $m[1];         // Correct blockname
            }

            //  Show this menu block
            if ($blockname == 'NOSHOW') { $noshow = 1; }
            if ($html) { print $html . $blockbottom; } // If started, finish it
            $html = $blocktop;
            if (! $noshow) {
                $html .= "<tr><td valign=\"middle\" class=\"menu-top\">&nbsp;" .
                    $BLOCKNAMEHTML . $blockname . "</td></tr>\n";
            }
            continue;
        }
        if (! $blockname) { continue; }     // No play if no block found

        //  Maybe insert a little spacer for looks
        if (preg_match('/^BLANKLINE/', $line)) {
            $html .= "<tr><td class=\"menus-blank\">&nbsp;</td></tr>\n";
            continue;
        }

        //  This is a bullet item
        if (preg_match ("/^<a+.*href=/", $line)) {
            $html .= "<tr><td class=\"menu-item\">\n" .
                "  &nbsp;$MENUITEMHTML$line" .
                "</td></tr>\n";
            continue;
        }

        //  This is an indented bullet item
        if (preg_match ("/^INDENT<a+.*href=/", $line)) {
            $line = substr($line,6);
            $html .= "  <tr><td class=\"menu-item\">\n" .
                "  &nbsp;$MENUINDENTHTML$line" .
                "  </td></tr>\n";
            continue;
        }

        //  What remains better be a set of lines of the form <key ... </key
        if (! preg_match ("/^<(\w+)/", $line, $m)) {
            redmsg("Invalid menu line $n in '$file': $line");
            continue;
        }
        $startn = $n;
        $key = $m[1];                   // Take all lines up until </key
        $h = $line;
        for ($n++; $n<count($lines); $n++) {
            $line = $lines[$n];
            $h .= $line;
            if (preg_match ("/^<\/$key/", $line)) { break; }
        }
        if (! preg_match ("/^<\/$key/", $line)) {
            redmsg("Menu block '$key' started at $startn not terminated in '$file'");
            continue;
        }
        $html .= "  <tr><td class=\"menu-item\">\n$h\n</td></tr>\n";
    }
    if ($html) { print $html . $blockbottom; } // If started, finish it
    print "<!-- End menu $file -->\n";
}

/*---------------------------------------------------------------
# do_center - Read a file and insert it in the center section
#
# Parameters:
#   file - option file to read. If not provided, the file to
#           insert will be derived from $_SERVER[PHP_SELF]
#           with .htm or .html appended.
#
# Note:
#   Title line in file of form TITLE=string
#   Lines in file of form KEYWORDS=w1 w2 w3 are extracted
#   Footer line in file of form FOOTER=string
#   Style import line in file of form STYLE=@import url("/sws/styleFUSION.css");
#
# Returns:
#   html
#   keywords from KEYWORDS are generated in header
---------------------------------------------------------------*/
function do_center($file = '') {
    global $REVISIONDATE, $_SERVER, $TITLE, $FOOTER, $STYLE;

    //  If no file provided, derive one
    if (! $file) {
        $file = $_SERVER['PHP_SELF'];
        if (! preg_match('/\/([^\/]+)\.php$/', $file, $m)) {
            redmsg("do_center: Unable to derive file to read from '$file'");
            return array('','','');
        }
        $file = $m[1] . '.htm';
        if (! file_exists($file)) { $file = $m[1] . '.html'; }
        if (! file_exists($file)) {
            redmsg("do_center: No HTML file found for " . $_SERVER['PHP_SELF'] . "'");
            return array('','');
        }
    }

    //  Just read and print file to STDOUT
   $html = "<!-- Centerfile $file -->\n";
    $kw = '';
    if (file_exists($file)) {
        $REVISIONDATE = "<br/>Last Revision: " . date('M d, Y', filemtime($file));
        $lines = file($file);
        for ($i=0; $i<count($lines); $i++) {
            if (preg_match ('/^<\!--\s*KEYWORDS=\s*(.+)\s*-->\s*$/', $lines[$i], $m)) {
                $kw .= $m[1] . ' ';
                continue;
            }
            if (preg_match ('/^<\!--\s*TITLE=\s*(.+)\s*-->\s*$/', $lines[$i], $m)) {
                $TITLE = $m[1];
                continue;
            }
            if (preg_match ('/^<\!--\s*FOOTER=\s*(.+)\s*-->\s*$/', $lines[$i], $m)) {
                $FOOTER = $m[1];
                continue;
            }
            if (preg_match ('/^<\!--\s*STYLE=\s*(.+)\s*-->\s*$/', $lines[$i], $m)) {
                $STYLE = "<style type=\"text/css\">$m[1]</style>\n";
                continue;
            }
            $html .= $lines[$i];
        }
    }
    else { $html .= "<font color=red>File '$file' not found</font>\n"; }
    $html .= "<!-- End centerfile $file -->\n";
    return array($html, $kw);
}
    
/*---------------------------------------------------------------
# do_footer - Generate footer
#
# Parameters:
#   none            Will use GLOBAL variable $FOOTER
#
# Returns:
#   nothing
---------------------------------------------------------------*/
function do_footer() {
    global $CONTACT, $REVISIONDATE, $SITENAME, $COPYRIGHTYEAR, $FOOTER;

    //  If $FOOTER is provided, use that
    if ($FOOTER) { print $FOOTER . "\n</body></html>\n"; return; }

    $copyryr = date('Y');
    if ($copyryr != $COPYRIGHTYEAR) { $copyryr = "$COPYRIGHTYEAR-$copyryr"; }

    print <<<END
<div class="footer">
<hr size="4" width="80%" noshade="noshade"/>
Copyright (c) $copyryr, $SITENAME<br/>
Report problems to <a class="footer" href="mailto:$CONTACT">$CONTACT</a><br/>
$REVISIONDATE <br/>
</div>
</body></html>

END;
}

/*---------------------------------------------------------------
# redmsg - Displays message in red
#
# Parameters:
#   msg - message to issue
#
# Returns:
#   nothing
---------------------------------------------------------------*/
function redmsg($msg) {

    print "<br/><font color=\"#ff0000\"><b>$msg</b></font><br/>\n";
}

/*---------------------------------------------------------------
# findfile - Return the HTML for a file if it exists
#
# Parameters:
#   f - file to read. May be null. Must be filename (without
#       extension and no directory). File in current working
#       directory only.
#
# Returns:
#   html
---------------------------------------------------------------*/
function findfile($f) {
    global $REVISIONDATE, $KEYWORDS, $TITLE, $FOOTER;

    //  If empty, nothing to do
    if (! $f) { return ''; }

    //  Be sure this is a simple filename
    if (preg_match('/^[^\/]+$/', $f)) {
        if (! $_SERVER) { $me = $HTTP_SERVER_VARS["SCRIPT_FILENAME"]; }
        else { $me = $_SERVER["SCRIPT_FILENAME"]; }
        $p = strrpos($me, '/');         // Work around broken PHP sites
        $m = substr($me,0,$p);
        $file = "$m/$f.html";
        if (! file_exists($file)) { $file = "$m/$f.htm"; }
        if (file_exists($file)) {
            $lines = file($file);
            $REVISIONDATE = "<br/>Last Revision: " . date('M d, Y', filemtime($file));
            $html = '';
            for ($i=0; $i<count($lines); $i++) {
                if (preg_match ('/^<\!--\s*KEYWORDS=\s*(.+)\s*-->\s*$/', $lines[$i], $m)) {
                    $KEYWORDS .= ' ' . $m[1] . "\n";
                    continue;
                }
                if (preg_match ('/^<\!--\s*TITLE=\s*(.+)\s*-->\s*$/', $lines[$i], $m)) {
                    $TITLE = $m[1];
                    continue;
                }
                if (preg_match ('/^<\!--\s*FOOTER=\s*(.+)\s*-->\s*$/', $lines[$i], $m)) {
                    $FOOTER = $m[1];
                    continue;
                }
                $html .= $lines[$i];
            }
            return $html;
        }
    }
    
    return "<br/><font color=\"#ff0000\"><b>Invalid FILE: $f</b></font>\n";
}

/*---------------------------------------------------------------
# session - Does the PHP session retrieval. If anything is 
#           being saved in a session, some other program did it all.
#           This assumes a session of the name we use was set up.
#
# Parameters:
#   setup - optional boolean if a sesssion is to be set up or not
#
# Returns:
#   nothing
---------------------------------------------------------------*/
function session($setup = 0) {
    global $EXPIRATION_PERIOD, $SESSIONCOOKIENAME, $_SERVER, $_SESSION;

    //  Watch for authorization to expire
    ini_set('session.use_trans_sid', 0);    // Stop adding SID to URLs
    ini_set('session.use_cookies', 1);      // Use cookie to store the session ID
    ini_set('session.name', $SESSIONCOOKIENAME);    // Set name of our cookie
    session_cache_limiter ('private, must-revalidate');

    //  If we are not to set up a session, just check for time to expire
    if (! $setup) {
        if (! session_id()) { session_start(); }
        if (! isset($_SESSION['SESSTIME'])) { $_SESSION['SESSTIME'] = 0; }
        $exp = $_SESSION['SESSTIME'] - time();
        if ( $exp < 0) {            // If session expired, clean up
            unset($_SESSION['SESSTIME']);
            unset($_SESSION['REMOTE_USER']);
        }
        else {                      // We have a session, call user override
            if (function_exists('CONFIG_OVERRIDE')) { CONFIG_OVERRIDE($_SESSION['REMOTE_USER']); }
        }
        return;
    }

    // Set up a new session, set expiration time
    $t = time();
    session_id($t);
    session_start();
    $SESSTIME = $t + $EXPIRATION_PERIOD;        // We are authorized until this time
    $_SESSION['SESSTIME'] = $SESSTIME;
    session_register ('SESSTIME');              // Actually creates session
    $_SESSION['REMOTE_USER'] = $_SERVER['REMOTE_USER'];
    session_register ('REMOTE_USER');           // Useless unless /private requires pw
    // We have a session, call user override
    if (function_exists('CONFIG_OVERRIDE')) { CONFIG_OVERRIDE($_SESSION['REMOTE_USER']); }
}

/*---------------------------------------------------------------
# isolate_parms - Given a list of parameter keys, build an
#       array of values taken from POST, GET or SESSION data.
#
# Recommended call:  extract(isolate_parms('a','b','c'));
#
# Parameters:
#   key1, key2, ... keyN - variables we want set
#
# Returns:
#   array for extract
---------------------------------------------------------------*/
function isolate_parms() {
    global $_POST, $_GET, $_SESSION;

    $parmvars = func_get_args();        // Get arguments to me as an array
    $myvars = array();
    while (list ($key, $val) = each($parmvars)) {
        $v = '';
        if (isset($_POST[$val])) { $v=$_POST[$val]; }
        elseif (isset($_GET[$val])) { $v=$_GET[$val]; }
        elseif (isset($_SESSION[$val])) { $v=$_SESSION[$val]; }
        $myvars[$val]=$v;
    }
    return $myvars;
}

?>
