<?php
/*#################################################################
#
# Name:	showfilelist.php
#
#   Use this to show an annotated list of files within the context
#   of an SWS web site.
#
# Parameters:
#   dir - directory where file lives. Relative to DOCUMENT_ROOT
#   file - file containing a list of files to show
#
# Format of '$file' is a sequence of key=values where KEY may be:
#
#   pagetitle=string     Becomes title for HTML page
#
#   header=string        Page header (enclosed in a <h3></h3> tag)
#
#   section=string       Section header (enclosed in a <h3></h3> tag)
#
#   item=string          Starts a new link (enclosed in <li></li> tag)
#   item2=string         Starts a second level new link (enclosed in <li></li> tag)
#   url=string           Optional URL for <a href></a> tag
#   comment=string       Optional comments added after <a href></a> tag
#
#   Jul 23, 2004  tpg      Initial coding begun
#   Jan 20, 2005  tpg      Introduce new format for input file
#
# Copyright (C) 2004-2005 Terry Gliedt
# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; See http://www.gnu.org/copyleft/gpl.html
#################################################################*/
//ini_set('error_reporting', E_ALL ^ E_NOTICE);
$SWSDIR = $_SERVER['DOCUMENT_ROOT'] . '/sws';   // Where all my files are
include_once "$SWSDIR/common.php";

session();                          //  Watch for authorization to expire

//  Get parameters passed in via normal invocation
extract(isolate_parms( 'dir', 'file'));

list($title,$html) = showfilelist($dir, $file);
do_page($title, $html);

exit;


/*#################################################################
#
# Name:	showfilelist($dir, $file)
#
#   Show an HTML list of lines in a simple file.
#
# Parameters:
#   dir - name of directory relative to HTDOCS
#   file - file in $dir
#
# Returns:
#   title of HTML
#   string of HTML
#
#################################################################*/
function showfilelist ($dir, $file) {
    global $SWSDIR, $REVISIONDATE;
    $pagetitle = '';                    // Must have an HTML page title
    $html = '';
    $openitem = "<ul>\n";               // HTML to open/close lists
    $closeitem = "</ul>\n";  
    $openitem2 = "<ul>\n";  
    $closeitem2 = "</ul>\n";  

    //  Remove attempts to screw with path
    $dir = str_replace('..','bogus',$dir);

    //  Deal with old broken PHP sites
    $p = strrpos($SWSDIR, '/');
    $documentroot = substr($SWSDIR,0,$p);

    //  Read file of URLs
    $f = "$documentroot/$dir/$file";
    if ($ff = file($f)) {
        $REVISIONDATE = "<br>Last Revision: " . date('M d, Y', filemtime($f));
    }

    $title = '';                    // Required
    $url = '';                      // Optional
    $comment = '';                  // Optional
    $iteminprogress = 0;            // If set, we have a item list going
    $item2inprogress = 0;
    $item = '';
    $item2 = '';
    while (list ($line_num, $line) = each($ff)) {
        if (preg_match ("/^\s*$/", $line)) { continue; }

        //  For old files, the first line will be both kinds of title
        if ($line_num == 0 && ! preg_match("/^#/", $line)) {
            $pagetitle = $line;
            $html .= "<h3>$line</h3>\n";
            continue;
        }

        //  Page title or for old files, the first line
        if (preg_match ("/^pagetitle=(.+)\s*$/", $line, $m)) {
            $pagetitle = $m[1];
            continue;
        }
        //  title=string
        if (preg_match ("/^header=(.+)\s*$/", $line, $m)) {
            $html .= "<h3>$m[1]</h3>\n";
            continue;
        }
        //  section=string
        if (preg_match ("/^section=(.+)\s*$/", $line, $m)) {
            if ($item2) { $html .= builditem($item2, $url, $comment); $item2 = ''; }
            if ($item2inprogress) { $item2inprogress = 0; $html .= $closeitem2; }
            if ($item)  { $html .= builditem($item, $url, $comment); $item = ''; }
            if ($iteminprogress)  { $iteminprogress = 0;  $html .= $closeitem; }
            $html .= "<h4>$m[1]</h4>\n";
            continue;
        }
        //  item=string
        if (preg_match ("/^item=(.+)\s*$/", $line, $m)) {
            if ($item2) { $html .= builditem($item2, $url, $comment); $item2 = ''; }
            if ($item2inprogress)  { $item2inprogress = 0; $html .= $closeitem2; }
            if (! $iteminprogress) { $iteminprogress = 1;  $html .= $openitem; }
            if ($item)  { $html .= builditem($item, $url, $comment); $item = ''; }
            $item = $m[1];
            $url = '';
            $comment = '';
            continue;
        }
        //  item2=string
        if (preg_match ("/^item2=(.+)\s*$/", $line, $m)) {
            if ($item)  { $html .= builditem($item, $url, $comment); $item = ''; }
            if (! $item2inprogress) { $item2inprogress = 1; $html .= $openitem2; }
            if ($item2) { $html .= builditem($item2, $url, $comment); $item2 = ''; }
            $item2 = $m[1];
            $url = '';
            $comment = '';
            continue;
        }
        //  url=string
        if (preg_match ("/^url=(.+)\s*$/", $line, $m)) {
            $url = $m[1];
            $c = substr($url,0,1);          // Check for relative path
            $c4 = substr($url,0,4);
            if ($c != '/' && $c4 != 'http') { $url = "/$dir/$url"; }
            continue;
        }
        //  comment=string
        if (preg_match ("/^comment=(.+)\s*$/", $line, $m)) {
            $comment = $m[1];
            continue;
        }

        //  Old format is still supported
        if (preg_match ("/^# section\s+(.+)/", $line, $m)) {
            if ($iteminprogress)  { $iteminprogress = 0;  $html .= $closeitem; }
            $html .= "<h4>$m[1]</h4>\n";
            continue;
        }
        if (preg_match ("/^#/", $line)) { continue; }
        if (preg_match("/^(\S+)\s+(.+)/", $line, $m)) {
            if (! $iteminprogress) { $iteminprogress = 1; $html .= $openitem; }
            if ($item) { $html .= builditem($item, $url, $comment); }
            $url = $m[1];
            $item = $m[2];
            $c = substr($url,0,1);          // Check for relative path
            $c4 = substr($url,0,4);
            if ($c != '/' && $c4 != 'http') { $url = "/$dir/$url"; }
            $html .= builditem($item, $url, '');
            $item = '';                     // Start new item
            $item2 = '';
            $url = '';
            $comment = '';
            continue;
        }

        $html .= "<b>Unable to parse line $line_num of '$file'</b><br>\n";
    }

    //  Finish any item in progress, finish item lists
    if ($item)  { $html .= builditem($item, $url, $comment); }
    if ($item2) { $html .= builditem($item2, $url, $comment); }
    if ($item2inprogress) { $html .= $closeitem2; }
    if ($iteminprogress) { $html .= $closeitem; }

    //  We must have seen pagetitle - our check for a reasonable file
    if (! $pagetitle) {
        return array('Illegal file', 'Never found "pagetitle keyword"');
    }

    $html .= "<br><br>\n";
    return array($pagetitle, $html);
}

/*#################################################################
#
# Name:	builditem($item, $url, $comment)
#
#################################################################*/
function builditem ($item, $url, $comment) {
    $h = "<li>";
    if ($url) { $h .= "<a href=\"$url\">"; }
    $h .= "<b>$item</b>";
    if ($url) { $h .= "</a>"; }
    if ($comment) { $h .= " $comment"; }
    $h .= "</li>\n";
    return $h;
}

?>
